/*
* ASIX FORTE serial EEPROM 93LC46B example
* distributed as-is, without any warranties
* http://www.asix.net/
*
* version 1.0, Dec 2016
* 
* Programmer connection table: 
* 93LC46B <->  FORTE
* CS      <->  P
* VCC     <->  VDD
* VSS     <->  GND
* DI      <->  D
* CLK     <->  C
* DO      <->  I
*
*/

#include <stdio.h>
#include "forte.h"

void ReadStatus(char* message)
{
    int status = AGetBlocking();
    char* codestring = ReturnCodeString(status);
    if(message[0] != '\0') {
        printf("%s: %s\n", message, codestring);
    }
    else {
        printf("%s\n", codestring);
    }
}

void PrintHex(int* data, int count)
{
    int i;
    for(i=0;i<count;i++) {
        printf("%02X ", data[i]);
        if(((i+1)%16) == 0) printf("\n");
    }
    if((count%16) != 0) printf("\n");
}

unsigned char LsbToMsb(unsigned char databyte)
{
    int i;
    unsigned char x = 0;
    for(i=0;i<8;i++) {
        x = x | (((databyte >> i) & 1) << (7-i));
    }
    return x;
}

int main(int argc, char *argv[])
{
    int i, count, status, received_bytes;
    int data[128];
    
    printf("ASIX FORTE serial EEPROM 93LC46B example \n");
   
    if(LoadForteDll("forte.dll") == FORTE_DLL_LOAD_OK) {
        printf("DLL load OK\n");
    }
    else {
        printf("DLL load FAILED\n");
        return 1;
    }
    
    QOpenProg(-1);
    ReadStatus("");
    
    // initiate
    QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");
    QPoweronVdd(20000, 3300); // supply voltage 3.3V
    ReadStatus("PoweronVdd");
    QSetShiftSpeed(SHIFT_CLK_1000kHz);
    QSetPins((PINS_LO << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)); // CS=0, CLK=0, DI=0
    QDelay(1);
    QSetPins((PINS_HI << PINS_P_BIT)); // CS=1
    QDelay(1);
    
    // 9-bit command
    QShiftBits(0x003, SHIFT_MODE0, 9);
    // read memory
    count = 128; // bytes
    QShiftBytes_In(SHIFT_MODE1, SHIFT_OUTIN_PINI, count);
    QSetPins((PINS_LO << PINS_P_BIT)); // CS=0
    // get data
    while(1) {
        status = AGetStatus(&received_bytes);
        if(!status) {
            count = 0;
            break;
        }
        if(received_bytes >= count) {
            AGetBlock(data, count, &received_bytes);
            break;
        }
    }
    
    // take 8-bit data and print
    for(i=0;i<count;i++) data[i] = LsbToMsb(data[i] & 0xFF);
    printf("Data:\n");
    PrintHex(data, count);
    
    QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");
    QCloseProg();
    ReadStatus("");
    
    return 0;
}

