/////////////////////////////////////////////////////////////////////////////
// Name:        childframes.cpp
// Purpose:     floating windows
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "childframes.h"

// ----------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// ----------------------------------------------------------------------------

// the event tables connect the wxWidgets events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.

BEGIN_EVENT_TABLE(LogWindow, wxMDIChildFrame)
    EVT_CLOSE(LogWindow::OnCloseQuery)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(DeviceHelpWindow, wxMDIChildFrame)
    EVT_CLOSE(DeviceHelpWindow::OnCloseQuery)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MemoryWindow, wxMDIChildFrame)
    EVT_CLOSE(MemoryWindow::OnCloseQuery)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(SettingsWindow, wxMDIChildFrame)
    EVT_CLOSE(SettingsWindow::OnCloseQuery)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(BitAliasWindow, wxMDIChildFrame)
    EVT_CLOSE(BitAliasWindow::OnCloseQuery)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(ActionWindow, wxMDIChildFrame)
    EVT_CLOSE(ActionWindow::OnCloseQuery)
    EVT_SIZE(ActionWindow::OnSize)
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class LogWindow
// ----------------------------------------------------------------------------

// frame constructor
LogWindow::LogWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi)
       : wxMDIChildFrame(parent, -1, title, pos, size, wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX|wxCAPTION)
{
    txt = new wxTextCtrl(this,-1,_T(""),wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_READONLY|wxHSCROLL);
    mi = check_mi;
    SetIcon(wxICON(ICON_LOG));
}

LogWindow::~LogWindow()
{
    delete txt;
}

void LogWindow::OnCloseQuery(wxCloseEvent& event)
{
    event.Veto();
    this->Hide();
    mi->Check(false);
}

void LogWindow::AddLine(wxString line)
{
    txt->AppendText(line+'\n');
}

void LogWindow::Clear()
{
    txt->Clear();
}

void LogWindow::SaveToFile(wxString filename)
{
    txt->SaveFile(filename);
}

// ----------------------------------------------------------------------------
// class DeviceHelpWindow
// ----------------------------------------------------------------------------

// frame constructor
DeviceHelpWindow::DeviceHelpWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi)
       : wxMDIChildFrame(parent, -1, title, pos, size, wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX|wxCAPTION)
{
    txt = new wxTextCtrl(this,-1,_T(""),wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_READONLY|wxHSCROLL);
    mi = check_mi;
    SetIcon(wxICON(ICON_HLP));
}

DeviceHelpWindow::~DeviceHelpWindow()
{
    delete txt;
}

void DeviceHelpWindow::OnCloseQuery(wxCloseEvent& event)
{
    event.Veto();
    this->Hide();
    mi->Check(false);
}

void DeviceHelpWindow::AddLine(wxString line)
{
    txt->AppendText(line+'\n');
}

// ----------------------------------------------------------------------------
// class MemoryWindow
// ----------------------------------------------------------------------------

// frame constructor
MemoryWindow::MemoryWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size,
                           wxMenuItem *check_mi, int wordsize, int wordcount)
       : wxMDIChildFrame(parent, -1, title, pos, size, wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX|wxCAPTION)
{
    mi = check_mi;
    mem_wordsize = wordsize;
    mem_words = wordcount;
    editor = new HexEdit(this, -1, wordcount, wordsize);
    SetIcon(wxICON(ICON_MEM));
}

MemoryWindow::~MemoryWindow()
{
    delete editor;
}

void MemoryWindow::OnCloseQuery(wxCloseEvent& event)
{
    event.Veto();
    this->Hide();
    mi->Check(false);
}

void MemoryWindow::SetCallback(wxObject* dst, EditorUpdateHandler handler)
{
    editor->SetCallback(dst, handler);
}

void MemoryWindow::GetData(int word, unsigned char* data)
{
    for(int i=0;i<mem_wordsize;i++) data[mem_wordsize-1-i]=editor->memory[word*mem_wordsize+i];
}

void MemoryWindow::SetData(int word, unsigned char* data)
{
    for(int i=0;i<mem_wordsize;i++) editor->memory[word*mem_wordsize+i]=data[mem_wordsize-1-i];
}

void MemoryWindow::Repaint()
{
    editor->Repaint();
}

// ----------------------------------------------------------------------------
// class SettingsWindow
// ----------------------------------------------------------------------------

// frame constructor
SettingsWindow::SettingsWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi)
       : wxMDIChildFrame(parent, -1, title, pos, size, wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX|wxCAPTION)
{
    settings = new wxPropertyGrid(this,-1,wxDefaultPosition,wxDefaultSize,wxPG_SPLITTER_AUTO_CENTER | wxPG_DEFAULT_STYLE);
    mi = check_mi;
    SetIcon(wxICON(ICON_SET));
}

SettingsWindow::~SettingsWindow()
{
    delete settings;
}

void SettingsWindow::OnCloseQuery(wxCloseEvent& event)
{
    event.Veto();
    this->Hide();
    mi->Check(false);
}

// ----------------------------------------------------------------------------
// class BitAliasWindow
// ----------------------------------------------------------------------------

// frame constructor
BitAliasWindow::BitAliasWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi, int evt_id)
       : wxMDIChildFrame(parent, -1, title, pos, size, wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX|wxCAPTION)
{
    aliases = new wxPropertyGrid(this,evt_id,wxDefaultPosition,wxDefaultSize,wxPG_SPLITTER_AUTO_CENTER | wxPG_DEFAULT_STYLE);
    mi = check_mi;
    SetIcon(wxICON(ICON_ALI));
}

BitAliasWindow::~BitAliasWindow()
{
    delete aliases;
}

void BitAliasWindow::OnCloseQuery(wxCloseEvent& event)
{
    event.Veto();
    this->Hide();
    mi->Check(false);
}

// ----------------------------------------------------------------------------
// class ActionWindow
// ----------------------------------------------------------------------------

// frame constructor
ActionWindow::ActionWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi, int cols)
       : wxMDIChildFrame(parent, -1, title, pos, size, wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX|wxCAPTION)
{
    mi = check_mi;
    columns = cols;
    panel = new wxPanel(this,wxID_ANY,wxPoint(0,0),wxSize(50,10));
    SetMinClientSize(wxSize(50,10));
    SetIcon(wxICON(ICON_ACT));
}

ActionWindow::~ActionWindow()
{
    int i;
    for(i=0;i<(int)buttons.Count();i++) {
        delete (wxButton*)(buttons[i]);
    }
    delete panel;
}

void ActionWindow::OnCloseQuery(wxCloseEvent& event)
{
    event.Veto();
    this->Hide();
    mi->Check(false);
}

void ActionWindow::OnSize(wxSizeEvent& event)
{
    UpdateSizes();
}

void ActionWindow::SetLayout(int columns_count)
{
    if(columns_count > 0) columns = columns_count;
    UpdateSizes();
}

void ActionWindow::AddButton(wxString caption, wxWindowID id)
{
    wxButton *button;
    button = new wxButton(panel,id,caption);
    buttons.Add(button);
    UpdateSizes();
}

void ActionWindow::UpdateSizes()
{
    wxButton* btn;
    int i, c, r, rows;
    int winwidth, winheight;
    int btnwidth, btnheight;
    int gaph, gapv;
    if(buttons.Count() == 0) return;
    rows = buttons.Count() / columns;
    if((buttons.Count() % columns) != 0) rows++;
    this->GetClientSize(&winwidth, &winheight);
    panel->SetSize(winwidth, winheight);
    gaph = (winwidth / 6) / (columns+1);
    gapv = (winheight / 6) / (rows+1);
    btnwidth = ((winwidth - gaph) / columns) - gaph;
    if(btnwidth<1) btnwidth=1;
    btnheight = ((winheight - gapv) / rows) - gapv;
    if(btnheight<1) btnheight=1;
    c=0; r=0;
    for(i=0;i<(int)buttons.Count();i++) {
        btn = (wxButton*)(buttons[i]);
        btn->SetSize(btnwidth, btnheight);
        btn->SetPosition(wxPoint(gaph+c*(btnwidth+gaph), gapv+r*(btnheight+gapv)));
        c++;
        if((c % columns) == 0) {
            c = 0;
            r++;
        }
    }
}
