/////////////////////////////////////////////////////////////////////////////
// Name:        childframes.h
// Purpose:     floating windows
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

#ifndef _CHILDFRAMES_H
#define _CHILDFRAMES_H

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/mdi.h>
#include <wx/propgrid/propgrid.h>

// user header files
#include "hexedit.h"

// ----------------------------------------------------------------------------
// class LogWindow
// ----------------------------------------------------------------------------

class LogWindow : public wxMDIChildFrame
{
public:
    // ctor(s)
    LogWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi);
    ~LogWindow();
    // functions
    void OnCloseQuery(wxCloseEvent& event);
    void AddLine(wxString line);
    void Clear();
    void SaveToFile(wxString filename);
    // variables
private:
    // variables
    wxMenuItem *mi;
    wxTextCtrl *txt;
    // functions
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// class DeviceHelpWindow
// ----------------------------------------------------------------------------

class DeviceHelpWindow : public wxMDIChildFrame
{
public:
    // ctor(s)
    DeviceHelpWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi);
    ~DeviceHelpWindow();
    // functions
    void OnCloseQuery(wxCloseEvent& event);
    void AddLine(wxString line);
    // variables
private:
    // variables
    wxMenuItem *mi;
    wxTextCtrl *txt;
    // functions
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// class MemoryWindow
// ----------------------------------------------------------------------------

class MemoryWindow : public wxMDIChildFrame
{
public:
    // ctor(s)
    MemoryWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, 
        wxMenuItem *check_mi, int wordsize, int wordcount);
    ~MemoryWindow();
    // functions
    void OnCloseQuery(wxCloseEvent& event);
    void SetCallback(wxObject* dst, EditorUpdateHandler handler);
    void GetData(int word, unsigned char* data);
    void SetData(int word, unsigned char* data);
    void Repaint();
    // variables
private:
    // variables
    wxMenuItem *mi;
    HexEdit *editor;
    int mem_wordsize;
    int mem_words;
    // functions
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// class SettingsWindow
// ----------------------------------------------------------------------------

class SettingsWindow : public wxMDIChildFrame
{
public:
    // ctor(s)
    SettingsWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi);
    ~SettingsWindow();
    // functions
    void OnCloseQuery(wxCloseEvent& event);
    // variables
    wxPropertyGrid *settings;
private:
    // variables
    wxMenuItem *mi;
    // functions
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// class BitAliasWindow
// ----------------------------------------------------------------------------

class BitAliasWindow : public wxMDIChildFrame
{
public:
    // ctor(s)
    BitAliasWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi, int evt_id);
    ~BitAliasWindow();
    // functions
    void OnCloseQuery(wxCloseEvent& event);
    // variables
    wxPropertyGrid *aliases;
private:
    // variables
    wxMenuItem *mi;
    // functions
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// class ActionWindow
// ----------------------------------------------------------------------------

class ActionWindow : public wxMDIChildFrame
{
public:
    // ctor(s)
    ActionWindow(wxMDIParentFrame* parent, const wxString& title, const wxPoint& pos, const wxSize& size, wxMenuItem *check_mi, int cols);
    ~ActionWindow();
    // functions
    void OnCloseQuery(wxCloseEvent& event);
    void OnSize(wxSizeEvent &event);
    void SetLayout(int columns_count);
    void AddButton(wxString caption, wxWindowID id);
    // variables
    //
private:
    // variables
    wxMenuItem *mi;
    int columns;
    wxArrayPtrVoid buttons;
    wxPanel *panel;
    // functions
    void UpdateSizes();
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif /* _CHILDFRAMES_H*/
