/////////////////////////////////////////////////////////////////////////////
// Name:        device.cpp
// Purpose:     device base class
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "device.h"

#define DEFBRDR 4

#include <wx/arrimpl.cpp> // this is a magic incantation which must be done!

WX_DEFINE_OBJARRAY(ArrayOfUserHandlers);

// ============================================================================
// implementation
// ============================================================================
  
// ----------------------------------------------------------------------------
// class Device
// ----------------------------------------------------------------------------
// constructor
Device::Device()   
{
    logwindow = NULL;
    devicehelpwindow = NULL;
    settingswindow = NULL;
    bitaliaswindow = NULL;
    actionwindow = NULL;
    memorywindows = NULL;
    action_columns = 2;
}

// virtual methods

wxString Device::GetName()
{
    return _T(STR_DEVICE_DEFAULT_NAME);
}

wxString Device::GetShortHint()
{
    return _T("");
}

wxString Device::GetLongHint()
{
    return _T("");
}

int Device::GetMemoriesCount()
{
    return 1;
}

int Device::GetMemoryWordSize(int memory_index)
{
    return 1; // in bytes
}

int Device::GetMemoryWordCount(int memory_index)
{
    return 16;
}

int Device::GetMemoryOffset(int memory_index)
{
    int i, offset;
    offset = 0;
    for(i=0;i<memory_index;i++) {
        offset += GetMemoryWordSize(i)*GetMemoryWordCount(i);
    }
    return offset;
}

wxString Device::GetMemoryName(int memory_index)
{
    wxString s;
    if(GetMemoriesCount()==1) s.Printf(_T(STR_DEVICE_MEMORY_NAME));
    else s.Printf(_T(STR_DEVICE_MULTIMEMORY_NAME), memory_index);
    return s;
}

void Device::RegisterActions()
{
    // AddAction(...)
    // SetActionLayout(...)
}

void Device::RegisterBitAliases()
{
    // AddBitAlias(...)
}

void Device::RegisterSettings()
{
    // AddSetting(...)
}

void Device::NewFile()
{
    FillScreenBuffer(0);
    RefreshScreen();
}

void Device::OpenFile()
{
    int filter_index;
    wxString filename = wxFileSelectorEx(STR_OPEN_FILE_CAPTION, wxEmptyString, wxEmptyString, &filter_index,
        STR_OPEN_FILE_FILTER, wxFD_OPEN|wxFD_FILE_MUST_EXIST);
    if(!filename.empty()) {
        FillScreenBuffer(0);
        if(filter_index==0) OpenAsBinary(filename);
        else if(filter_index==1) OpenAsHex(filename);
        RefreshScreen();
    }
}

void Device::SaveFileAs()
{
    int filter_index;
    wxString filename = wxFileSelectorEx(STR_SAVE_FILE_CAPTION, wxEmptyString, wxEmptyString, &filter_index,
        STR_SAVE_FILE_FILTER, wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(!filename.empty()) {
        MemoryWindowsToScreenBuffer();
        if(filter_index==0) SaveAsBinary(filename);
        else if(filter_index==1) SaveAsHex(filename);
    }
}

// non-virtual methods

void Device::AddAction(wxString button_label, wxString menu_label, UserHandler user_handler)
{
    actionwindow_btnlabel.Add(button_label);
    actionwindow_mnulabel.Add(menu_label);
    actionwindow_funct.Add(user_handler);
}

int Device::GetActionCount()
{
    return (int)actionwindow_btnlabel.Count();
}

void Device::Action(int id)
{
    UserHandler user_handler;
    user_handler = actionwindow_funct[id];
    (*this.*user_handler)();
}

void Device::SetActionLayout(int column_count)
{
    action_columns = column_count;
}

void Device::AddBitAlias(wxString label, int memory_index, int word, int bit_start, int bit_count)
{
    if((memory_index<0) || (memory_index>=GetMemoriesCount())) return;
    if((word<0) || (word>=GetMemoryWordCount(memory_index))) return;
    if((bit_start<0) || (bit_start>=(GetMemoryWordSize(memory_index)*8))) return;
    if(bit_count<1) return;
    //
    int b_from, b_to, b_last;
    b_from = word*GetMemoryWordSize(memory_index)*8 + bit_start;
    b_to = b_from + bit_count - 1;
    b_last = GetMemoryWordCount(memory_index)*GetMemoryWordSize(memory_index)*8 - 1;
    if(b_to > b_last) return;
    //
    bitaliaswindow_mem.Add(memory_index);
    bitaliaswindow_word.Add(word);
    bitaliaswindow_bit.Add(bit_start);
    bitaliaswindow_len.Add(bit_count);
    wxStringProperty *p;
    p = new wxStringProperty(label, wxPG_LABEL, "0");
    bitaliaswindow_aliases.Add(p);
}

int Device::GetBitAliasCount()
{
    return (int)bitaliaswindow_aliases.Count();
}

void Device::BitAliasWindowToScreenBuffer(int index)
{
    if(GetBitAliasCount()<1) return;
    //
    int mem, word, bit, count;
    int i, j, digit;
    wxStringProperty *p;
    wxString pv;
    char* bits;
    //
    p = (wxStringProperty*)bitaliaswindow_aliases[index];
    pv = p->GetValue().GetString();
    mem = bitaliaswindow_mem[index];
    word = bitaliaswindow_word[index];
    bit = bitaliaswindow_bit[index];
    count = bitaliaswindow_len[index];
    if(count>((int)pv.Len()*4)) {
        bits = new char[count];
        for(i=0;i<count;i++) bits[i]=0;
    }
    else {
        bits = new char[pv.Len()*4];
        for(i=0;i<((int)pv.Len()*4);i++) bits[i]=0;
    }
    // string to bits
    for(i=0;i<(int)pv.Len();i++) {
        digit = ToHex(pv.GetChar(pv.Len()-1-i));
        if(digit<0) break;
        for(j=0;j<4;j++) bits[i*4+j] = (digit >> j) & 1;
    }
    // fill bits
    for(i=0;i<count;i++) {
        SetScreenBufferBit(mem, word, bit, bits[i]);
        bit++;
        if(bit>=(GetMemoryWordSize(mem)*8)) {
            bit=0;
            word++;
        }
    }
    delete bits;
}

void Device::ScreenBufferToBitAliasWindow()
{
    if(GetBitAliasCount()<1) return;
    //
    int ai, mem, word, bit, count;
    int i, j, digits, v;
    wxStringProperty *p;
    wxString pv, s;
    char* bits;
    //
    for(ai=0;ai<GetBitAliasCount();ai++) {
        mem = bitaliaswindow_mem[ai];
        word = bitaliaswindow_word[ai];
        bit = bitaliaswindow_bit[ai];
        count = bitaliaswindow_len[ai];
        bits = new char[count+3];
        for(i=0;i<count+3;i++) bits[i]=0;
        // buffer to to bits
        for(i=0;i<count;i++) {
            bits[i] = GetScreenBufferBit(mem, word, bit);
            bit++;
            if(bit>=(GetMemoryWordSize(mem)*8)) {
                bit=0;
                word++;
            }
        }
        // bits to string
        digits = count/4;
        if(count % 4 != 0) digits++;
        pv = "";
        for(i=0;i<digits;i++) {
            v = 0;
            for(j=0;j<4;j++) v |= bits[i*4+j] << j;
            s.Printf("%X", v);
            pv = s + pv;
        }
        delete bits;
        // fill string
        p = (wxStringProperty*)bitaliaswindow_aliases[ai];
        p->SetValueFromString(pv);
    }
}

void Device::MemoryWindowsToScreenBuffer()
{
    int i, j;
    unsigned char* tmp = new unsigned char[max_bytes_per_word];
    for(i=0;i<max_bytes_per_word;i++) tmp[i]=0;
    for(j=0;j<GetMemoriesCount();j++) {
        for(i=0;i<GetMemoryWordCount(j);i++) {
            memorywindows[j]->GetData(i,tmp);
            SetScreenBuffer(j,i,tmp);
        }
    }
    delete tmp;
}

void Device::ScreenBufferToMemoryWindows()
{
    int i, j;
    unsigned char* tmp = new unsigned char[max_bytes_per_word];
    for(j=0;j<GetMemoriesCount();j++) {
        for(i=0;i<GetMemoryWordCount(j);i++) {
            GetScreenBuffer(j,i,tmp);
            memorywindows[j]->SetData(i,tmp);
        }
        memorywindows[j]->Repaint();
    }
    delete tmp;
}

void Device::SetScreenBuffer(int memory_index, int word, unsigned char* data)
{
    int i;
    for(i=0;i<max_bytes_per_word;i++) {
        screen_buffer[memory_index*max_words_count*max_bytes_per_word+word*max_bytes_per_word+i] = data[i];
    }
}

void Device::GetScreenBuffer(int memory_index, int word, unsigned char* data)
{
    int i;
    for(i=0;i<max_bytes_per_word;i++) {
        data[i] = screen_buffer[memory_index*max_words_count*max_bytes_per_word+word*max_bytes_per_word+i];
    }
}

void Device::SetScreenBufferBit(int memory_index, int word, int bit, int bit_value)
{
    unsigned char* tmp = new unsigned char[max_bytes_per_word];
    GetScreenBuffer(memory_index, word, tmp);
    unsigned char mask = 1 << (bit%8);
    tmp[bit/8] &= mask^0xFF;
    if(bit_value) tmp[bit/8] |= mask;
    SetScreenBuffer(memory_index, word, tmp);
    delete tmp;
}

int Device::GetScreenBufferBit(int memory_index, int word, int bit)
{
    unsigned char* tmp = new unsigned char[max_bytes_per_word];
    GetScreenBuffer(memory_index, word, tmp);
    int bit_value = ((tmp[bit/8]) >> (bit%8)) & 1;
    delete tmp;
    return bit_value;
}

void Device::FillScreenBuffer(unsigned char value)
{
    int i;
    for(i=0;i<GetMemoriesCount()*max_words_count*max_bytes_per_word;i++) {
        screen_buffer[i]=value;
    }
}

void Device::SetHiddenBuffer(int memory_index, int word, unsigned char* data)
{
    int i;
    for(i=0;i<max_bytes_per_word;i++) {
        hidden_buffer[memory_index*max_words_count*max_bytes_per_word+word*max_bytes_per_word+i] = data[i];
    }
}

void Device::GetHiddenBuffer(int memory_index, int word, unsigned char* data)
{
    int i;
    for(i=0;i<max_bytes_per_word;i++) {
        data[i] = hidden_buffer[memory_index*max_words_count*max_bytes_per_word+word*max_bytes_per_word+i];
    }
}

void Device::HiddenBufferToScreenBuffer()
{
    int i;
    for(i=0;i<GetMemoriesCount()*max_words_count*max_bytes_per_word;i++) {
        screen_buffer[i]=hidden_buffer[i];
    }
}

void Device::ScreenBufferToHiddenBuffer()
{
    int i;
    for(i=0;i<GetMemoriesCount()*max_words_count*max_bytes_per_word;i++) {
        hidden_buffer[i]=screen_buffer[i];
    }
}

void Device::RefreshScreen()
{
    ScreenBufferToBitAliasWindow();
    ScreenBufferToMemoryWindows();
}

void Device::MemoryWindowsToBitAliasWindow()
{
    MemoryWindowsToScreenBuffer();
    ScreenBufferToBitAliasWindow();
    ScreenBufferToMemoryWindows();
}

void Device::BitAliasWindowToMemoryWindows(wxPGProperty *p)
{
    int i;
    for(i=0;i<(int)bitaliaswindow_aliases.Count();i++) {
        if(bitaliaswindow_aliases[i]==(void*)p) {
            BitAliasWindowToScreenBuffer(i);
            ScreenBufferToBitAliasWindow();
            ScreenBufferToMemoryWindows();
            break;
        }
    }
}

void Device::AddSetting(wxPGProperty *prop)
{
    settingswindow_props.Add(prop);
}

void Device::Open(wxMDIParentFrame *parent, wxFileConfig *cfg, wxMenu *mnuAction, wxMenu *mnuWindow, Programmer *programmer)
{
    parent_window = parent;
    config = cfg;
    menuAction = mnuAction;
    menuWindow = mnuWindow;
    prog = programmer;
    //
    int i;
    max_bytes_per_word = 0;
    max_words_count = 0;
    for(i=0;i<(int)GetMemoriesCount();i++) {
        if(max_bytes_per_word < GetMemoryWordSize(i)) max_bytes_per_word = GetMemoryWordSize(i);
        if(max_words_count < GetMemoryWordCount(i)) max_words_count = GetMemoryWordCount(i);
    }
    screen_buffer = new unsigned char[GetMemoriesCount()*max_words_count*max_bytes_per_word];
    hidden_buffer = new unsigned char[GetMemoriesCount()*max_words_count*max_bytes_per_word];
    for(i=0;i<GetMemoriesCount()*max_words_count*max_bytes_per_word;i++) {
        screen_buffer[i]=0;
        hidden_buffer[i]=0;
    }
    //
    CreateActionWindow();
    CreateBitAliasWindow();
    CreateDeviceHelpWindow();
    CreateLogWindow();
    CreateSettingsWindow();
    CreateMemoryWindows();
    ConnectWindows();
}

void Device::CreateActionWindow()
{
    int i;
    // create list
    actionwindow_btnlabel.Clear();
    actionwindow_mnulabel.Clear();
    actionwindow_funct.Clear();
    RegisterActions();
    if(GetActionCount()<1) return;
    // add to menu
    menuWindow->Append(Menu_ShowAction, _T(STR_MENU_ACTION),_T(""),wxITEM_CHECK);
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowAction);
    bool win_s = config->ReadBool("/"+GetName()+"/ActionVisible", true);
    mi->Check(win_s);
    // create window
    int default_x = (parent_window->GetClientSize().GetWidth() / 3)+DEFBRDR;
    int default_y = DEFBRDR;
    int default_w = (parent_window->GetClientSize().GetWidth() / 3)-DEFBRDR*2;
    int default_h = (parent_window->GetClientSize().GetHeight() / 4)-DEFBRDR*2;
    long win_x = config->ReadLong("/"+GetName()+"/ActionPositionX", default_x);
    long win_y = config->ReadLong("/"+GetName()+"/ActionPositionY", default_y);
    long win_w = config->ReadLong("/"+GetName()+"/ActionWidth", default_w);
    long win_h = config->ReadLong("/"+GetName()+"/ActionHeight", default_h);
    actionwindow = new ActionWindow(parent_window, STR_WINDOW_ACTION, wxPoint(win_x,win_y), wxSize(win_w,win_h), mi, action_columns);
    if(win_s) actionwindow->Show();
    else actionwindow->Hide();
    // add actions to menu and add buttons
    for(i=0;i<GetActionCount();i++) {
        menuAction->Append(Menu_ActionList+i,actionwindow_mnulabel[i]);
        actionwindow->AddButton(actionwindow_btnlabel[i], Menu_ActionList+i);
    }
}

void Device::CreateBitAliasWindow()
{
    int i;
    // create list
    bitaliaswindow_aliases.Clear();
    bitaliaswindow_mem.Clear();
    bitaliaswindow_word.Clear();
    bitaliaswindow_bit.Clear();
    bitaliaswindow_len.Clear();
    RegisterBitAliases();
    if(GetBitAliasCount()<1) return;
    // add to menu
    menuWindow->Append(Menu_ShowBitAlias, _T(STR_MENU_BITALIAS),_T(""),wxITEM_CHECK);
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowBitAlias);
    bool win_s = config->ReadBool("/"+GetName()+"/BitAliasVisible", true);
    mi->Check(win_s);
    // create window
    int default_x = (parent_window->GetClientSize().GetWidth() / 3)+DEFBRDR;
    int default_y = (parent_window->GetClientSize().GetHeight() / 4)*2+DEFBRDR;
    int default_w = (parent_window->GetClientSize().GetWidth() / 3)-DEFBRDR*2;
    int default_h = (parent_window->GetClientSize().GetHeight() / 4)-DEFBRDR*2;
    long win_x = config->ReadLong("/"+GetName()+"/BitAliasPositionX", default_x);
    long win_y = config->ReadLong("/"+GetName()+"/BitAliasPositionY", default_y);
    long win_w = config->ReadLong("/"+GetName()+"/BitAliasWidth", default_w);
    long win_h = config->ReadLong("/"+GetName()+"/BitAliasHeight", default_h);
    bitaliaswindow = new BitAliasWindow(parent_window, STR_WINDOW_BITALIAS, wxPoint(win_x,win_y), wxSize(win_w,win_h), mi, Menu_BitAlias);
    if(win_s) bitaliaswindow->Show();
    else bitaliaswindow->Hide();
    // fill settings
    for(i=0;i<GetBitAliasCount();i++) {
        bitaliaswindow->aliases->Append((wxPGProperty *)bitaliaswindow_aliases[i]);
    }
}

void Device::CreateDeviceHelpWindow()
{
    wxString helpstr = GetLongHint();
    if(helpstr.IsEmpty()) return;  // do not create window if no help is available 
    // add to menu
    menuWindow->Append(Menu_ShowDeviceHelp, _T(STR_MENU_DEVICEHELP),_T(""),wxITEM_CHECK);
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowDeviceHelp);
    bool win_s = config->ReadBool("/"+GetName()+"/HelpVisible", true);
    mi->Check(win_s);
    // create window
    int default_x = (parent_window->GetClientSize().GetWidth() / 3)*2+DEFBRDR;
    int default_y = DEFBRDR;
    int default_w = (parent_window->GetClientSize().GetWidth() / 3)-DEFBRDR*3;
    int default_h = (parent_window->GetClientSize().GetHeight() / 2);
    long win_x = config->ReadLong("/"+GetName()+"/HelpPositionX", default_x);
    long win_y = config->ReadLong("/"+GetName()+"/HelpPositionY", default_y);
    long win_w = config->ReadLong("/"+GetName()+"/HelpWidth", default_w);
    long win_h = config->ReadLong("/"+GetName()+"/HelpHeight", default_h);
    devicehelpwindow = new DeviceHelpWindow(parent_window, STR_WINDOW_DEVICEHELP, wxPoint(win_x,win_y), wxSize(win_w,win_h), mi);
    devicehelpwindow->AddLine(helpstr);
    if(win_s) devicehelpwindow->Show();
    else devicehelpwindow->Hide();
}

void Device::CreateLogWindow()
{
    // add to menu
    menuWindow->Append(Menu_ShowLog, _T(STR_MENU_LOG),_T(""),wxITEM_CHECK);
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowLog);
    bool win_s = config->ReadBool("/"+GetName()+"/LogVisible", true);
    mi->Check(win_s);
    // create window
    int default_x = DEFBRDR;
    int default_y = (parent_window->GetClientSize().GetHeight() / 4)*3+DEFBRDR;
    int default_w = parent_window->GetClientSize().GetWidth()-DEFBRDR*3;
    int default_h = (parent_window->GetClientSize().GetHeight() / 4)-DEFBRDR*3;
    long win_x = config->ReadLong("/"+GetName()+"/LogPositionX", default_x);
    long win_y = config->ReadLong("/"+GetName()+"/LogPositionY", default_y);
    long win_w = config->ReadLong("/"+GetName()+"/LogWidth", default_w);
    long win_h = config->ReadLong("/"+GetName()+"/LogHeight", default_h);
    logwindow = new LogWindow(parent_window, STR_WINDOW_LOG, wxPoint(win_x,win_y), wxSize(win_w,win_h), mi);
    if(win_s) logwindow->Show();
    else logwindow->Hide();
}

void Device::CreateSettingsWindow()
{
    // create list
    settingswindow_props.Clear();
    RegisterSettings();
    if(settingswindow_props.IsEmpty()) return;
    // add to menu
    menuWindow->Append(Menu_ShowSettings, _T(STR_MENU_SETTINGS),_T(""),wxITEM_CHECK);
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowSettings);
    bool win_s = config->ReadBool("/"+GetName()+"/SettingsVisible", true);
    mi->Check(win_s);
    // create window
    int default_x = (parent_window->GetClientSize().GetWidth() / 3)+DEFBRDR;
    int default_y = (parent_window->GetClientSize().GetHeight() / 4)+DEFBRDR;
    int default_w = (parent_window->GetClientSize().GetWidth() / 3)-DEFBRDR*2;
    int default_h = (parent_window->GetClientSize().GetHeight() / 4)-DEFBRDR*2;
    long win_x = config->ReadLong("/"+GetName()+"/SettingsPositionX", default_x);
    long win_y = config->ReadLong("/"+GetName()+"/SettingsPositionY", default_y);
    long win_w = config->ReadLong("/"+GetName()+"/SettingsWidth", default_w);
    long win_h = config->ReadLong("/"+GetName()+"/SettingsHeight", default_h);
    settingswindow = new SettingsWindow(parent_window, STR_WINDOW_SETTINGS, wxPoint(win_x,win_y), wxSize(win_w,win_h), mi);
    if(win_s) settingswindow->Show();
    else settingswindow->Hide();
    // fill settings
    int i;
    for(i=0;i<(int)settingswindow_props.Count();i++) {
        wxPGProperty *prop;
        prop = (wxPGProperty *)settingswindow_props[i];
        settingswindow->settings->Append(prop);
        // load from config
        wxString s;
        config->Read("/"+GetName()+"/Option"+prop->GetName(), &s, prop->GetValueAsString(wxPG_FULL_VALUE));
        prop->SetValueFromString(s);
    }
}

void Device::CreateMemoryWindows()
{
    int i;
    wxString cfgkey;
    memorywindows = new MemoryWindow*[GetMemoriesCount()];
    for(i=0;i<GetMemoriesCount();i++) {
        cfgkey.Printf("Mem%d", i);
        cfgkey = "/"+GetName()+"/"+cfgkey;
        // add to menu
        menuWindow->Append(Menu_MemoryList+i,GetMemoryName(i),_T(""),wxITEM_CHECK);
        wxMenuItem *mi = menuWindow->FindChildItem(Menu_MemoryList+i);
        bool win_s = config->ReadBool(cfgkey+"Visible", true);
        mi->Check(win_s);
        // create window
        int default_x = DEFBRDR;
        int default_y = (((parent_window->GetClientSize().GetHeight()/4)*3)/GetMemoriesCount())*i+DEFBRDR;
        int default_w = (parent_window->GetClientSize().GetWidth() / 3)-DEFBRDR*2;
        int default_h = (((parent_window->GetClientSize().GetHeight()/4)*3)/GetMemoriesCount())-DEFBRDR*2;
        long win_x = config->ReadLong(cfgkey+"PositionX", default_x);
        long win_y = config->ReadLong(cfgkey+"PositionY", default_y);
        long win_w = config->ReadLong(cfgkey+"Width", default_w);
        long win_h = config->ReadLong(cfgkey+"Height", default_h);
        MemoryWindow* memorywindow = new MemoryWindow(parent_window, GetMemoryName(i), wxPoint(win_x,win_y),
            wxSize(win_w,win_h), mi, GetMemoryWordSize(i), GetMemoryWordCount(i));
        if(win_s) memorywindow->Show();
        else memorywindow->Hide();
        memorywindows[i] = memorywindow;
    }
}

void Device::ConnectWindows()
{
    if(bitaliaswindow==NULL) return;
    //
    int i;
    for(i=0;i<GetMemoriesCount();i++) {
        // change in memory to bit alias
        memorywindows[i]->SetCallback(this, (EditorUpdateHandler)&Device::MemoryWindowsToBitAliasWindow);
    }
}

void Device::Close()
{
    DestroyActionWindow();
    DestroyBitAliasWindow();
    DestroyDeviceHelpWindow();
    DestroyLogWindow();
    DestroySettingsWindow();
    DestroyMemoryWindows();
    //
    delete screen_buffer;
    delete hidden_buffer;
}

void Device::DestroyActionWindow()
{
    int i;
    if(actionwindow==NULL) return;
    // remove from menu
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowAction);
    menuWindow->Remove(mi);
    for(i=0;i<GetActionCount();i++) {
        // remove from menu
        wxMenuItem *mi = menuAction->FindChildItem(Menu_ActionList+i);
        menuAction->Remove(mi);
    }
    //
    wxRect win_xywh = actionwindow->GetRect();
    config->Write("/"+GetName()+"/ActionPositionX", win_xywh.x);
    config->Write("/"+GetName()+"/ActionPositionY", win_xywh.y);
    config->Write("/"+GetName()+"/ActionWidth", win_xywh.width);
    config->Write("/"+GetName()+"/ActionHeight", win_xywh.height);
    config->Write("/"+GetName()+"/ActionVisible", actionwindow->IsVisible());    
    delete actionwindow;
}

void Device::DestroyBitAliasWindow()
{
    if(bitaliaswindow==NULL) return;
    // remove from menu
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowBitAlias);
    menuWindow->Remove(mi);
    //
    wxRect win_xywh = bitaliaswindow->GetRect();
    config->Write("/"+GetName()+"/BitAliasPositionX", win_xywh.x);
    config->Write("/"+GetName()+"/BitAliasPositionY", win_xywh.y);
    config->Write("/"+GetName()+"/BitAliasWidth", win_xywh.width);
    config->Write("/"+GetName()+"/BitAliasHeight", win_xywh.height);
    config->Write("/"+GetName()+"/BitAliasVisible", bitaliaswindow->IsVisible());    
    delete bitaliaswindow;
}

void Device::DestroyDeviceHelpWindow()
{
    if(devicehelpwindow==NULL) return;
    // remove from menu
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowDeviceHelp);
    menuWindow->Remove(mi);
    //
    wxRect win_xywh = devicehelpwindow->GetRect();
    config->Write("/"+GetName()+"/HelpPositionX", win_xywh.x);
    config->Write("/"+GetName()+"/HelpPositionY", win_xywh.y);
    config->Write("/"+GetName()+"/HelpWidth", win_xywh.width);
    config->Write("/"+GetName()+"/HelpHeight", win_xywh.height);
    config->Write("/"+GetName()+"/HelpVisible", devicehelpwindow->IsVisible());    
    delete devicehelpwindow;
}

void Device::DestroyLogWindow()
{
    if(logwindow==NULL) return;
    // remove from menu
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowLog);
    menuWindow->Remove(mi);
    //
    wxRect win_xywh = logwindow->GetRect();
    config->Write("/"+GetName()+"/LogPositionX", win_xywh.x);
    config->Write("/"+GetName()+"/LogPositionY", win_xywh.y);
    config->Write("/"+GetName()+"/LogWidth", win_xywh.width);
    config->Write("/"+GetName()+"/LogHeight", win_xywh.height);
    config->Write("/"+GetName()+"/LogVisible", logwindow->IsVisible());    
    delete logwindow;
}

void Device::DestroySettingsWindow()
{
    if(settingswindow==NULL) return;
    // remove from menu
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowSettings);
    menuWindow->Remove(mi);
    //
    wxRect win_xywh = settingswindow->GetRect();
    config->Write("/"+GetName()+"/SettingsPositionX", win_xywh.x);
    config->Write("/"+GetName()+"/SettingsPositionY", win_xywh.y);
    config->Write("/"+GetName()+"/SettingsWidth", win_xywh.width);
    config->Write("/"+GetName()+"/SettingsHeight", win_xywh.height);
    config->Write("/"+GetName()+"/SettingsVisible", settingswindow->IsVisible());
    // save settings
    int i;
    for(i=0;i<(int)settingswindow_props.Count();i++) {
        wxPGProperty *prop;
        prop = (wxPGProperty *)settingswindow_props[i];
        wxString propname = prop->GetName();
        propname.Replace(" ", "_");
        config->Write("/"+GetName()+"/Option"+propname, prop->GetValueAsString(wxPG_FULL_VALUE));
    }
    delete settingswindow;
}

void Device::DestroyMemoryWindows()
{
    if(memorywindows==NULL) return;
    int i;
    wxString cfgkey;
    for(i=0;i<GetMemoriesCount();i++) {
        // remove from menu
        wxMenuItem *mi = menuWindow->FindChildItem(Menu_MemoryList+i);
        menuWindow->Remove(mi);
        //
        cfgkey.Printf("Mem%d", i);
        cfgkey = "/"+GetName()+"/"+cfgkey;
        wxRect win_xywh = memorywindows[i]->GetRect();
        config->Write(cfgkey+"PositionX", win_xywh.x);
        config->Write(cfgkey+"PositionY", win_xywh.y);
        config->Write(cfgkey+"Width", win_xywh.width);
        config->Write(cfgkey+"Height", win_xywh.height);
        config->Write(cfgkey+"Visible", memorywindows[i]->IsVisible());    
        delete memorywindows[i];
    }
    delete memorywindows;
}

void Device::ShowAll()
{
    ShowActionWindow(true);
    ShowBitAliasWindow(true);
    ShowDeviceHelpWindow(true);
    ShowLogWindow(true);
    ShowSettingsWindow(true);
    int i;
    for(i=0;i<GetMemoriesCount();i++) {
        ShowMemoryWindow(i,true);
    }
}

void Device::HideAll()
{
    ShowActionWindow(false);
    ShowBitAliasWindow(false);
    ShowDeviceHelpWindow(false);
    ShowLogWindow(false);
    ShowSettingsWindow(false);
    int i;
    for(i=0;i<GetMemoriesCount();i++) {
        ShowMemoryWindow(i,false);
    }
}

void Device::ShowActionWindow(bool show)
{
    if(actionwindow==NULL) return;
    if(show) actionwindow->Show();
    else actionwindow->Hide();
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowAction);
    mi->Check(show);
}

void Device::ShowBitAliasWindow(bool show)
{
    if(bitaliaswindow==NULL) return;
    if(show) bitaliaswindow->Show();
    else bitaliaswindow->Hide();
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowBitAlias);
    mi->Check(show);
}

void Device::ShowDeviceHelpWindow(bool show)
{
    if(devicehelpwindow==NULL) return;
    if(show) devicehelpwindow->Show();
    else devicehelpwindow->Hide();
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowDeviceHelp);
    mi->Check(show);
}

void Device::ShowLogWindow(bool show)
{
    if(logwindow==NULL) return;
    if(show) logwindow->Show();
    else logwindow->Hide();
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowLog);
    mi->Check(show);
}

void Device::ShowSettingsWindow(bool show)
{
    if(settingswindow==NULL) return;
    if(show) settingswindow->Show();
    else settingswindow->Hide();
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_ShowSettings);
    mi->Check(show);
}

void Device::ShowMemoryWindow(int memory_index, bool show)
{
    if(memorywindows==NULL) return;
    if(show) memorywindows[memory_index]->Show();
    else memorywindows[memory_index]->Hide();
    wxMenuItem *mi = menuWindow->FindChildItem(Menu_MemoryList+memory_index);
    mi->Check(show);
}

void Device::AddLog(wxString line)
{
    if(logwindow==NULL) return;
    wxDateTime now = wxDateTime::Now();
    logwindow->AddLine(now.Format("%X", wxDateTime::Local) + " " + line);
}

void Device::AddLog(wxString message, int value)
{
    if(logwindow==NULL) return;
    wxString line;
    line.Printf(" : %d (0x%X)", value, value);
    AddLog(message+line);
}

void Device::ClearLog()
{
    if(logwindow==NULL) return;
    logwindow->Clear();
}

void Device::SaveLog(wxString filename)
{
    if(logwindow==NULL) return;
    logwindow->SaveToFile(filename);
}

int Device::ToHex(wxUniChar hex)
{
	return ( hex >= '0' && hex <= '9' ) ? hex -'0' :
			( hex >= 'a' && hex <= 'f' ) ? hex -'a' + 10:
			( hex >= 'A' && hex <= 'F' ) ? hex -'A' + 10:
			-1;
}

void Device::ToStr(unsigned char data, unsigned char* buf)
{
    if((data & 0xF0) < 0xA0) buf[0]=(data>>4)+'0';
    else buf[0]=(data>>4)-0xA+'A';
    if((data & 0xF) < 0xA) buf[1]=(data&0xF)+'0';
    else buf[1]=(data&0xF)-0xA+'A';
}

void Device::SaveAsBinary(wxString filename)
{
    int m,w;
    unsigned char* tmp;
    wxFFileOutputStream out(filename);
    if(!(out.IsOk())) {
        wxMessageBox(_T(STR_SAVE_FILE_ERR_MESSAGE), _T(STR_SAVE_FILE_ERR_CAPTION), wxOK | wxICON_ERROR);
        return;
    }
    tmp = new unsigned char[max_bytes_per_word];
    for(m=0;m<GetMemoriesCount();m++) {
        for(w=0;w<GetMemoryWordCount(m);w++) {
            GetScreenBuffer(m,w,tmp);
            out.Write(tmp, GetMemoryWordSize(m));
        }
    }
    delete tmp;
    AddLog(STR_SAVE_OK + filename);
}

void Device::SaveAsHex(wxString filename)
{
    int m,w,i,pos,off;
    unsigned char* tmp;
    unsigned char* mem_bytes;
    unsigned char line[21];
    unsigned char crc;
    unsigned char b[2];
    int mem_bytes_count, bytes_to_write;
    wxFFileOutputStream out(filename);
    if(!(out.IsOk())) {
        wxMessageBox(_T(STR_SAVE_FILE_ERR_MESSAGE), _T(STR_SAVE_FILE_ERR_CAPTION), wxOK | wxICON_ERROR);
        return;
    }
    tmp = new unsigned char[max_bytes_per_word];
    off = 0;
    for(m=0;m<GetMemoriesCount();m++) {
        mem_bytes_count = GetMemoryWordCount(m)*GetMemoryWordSize(m);
        mem_bytes = new unsigned char[mem_bytes_count];
        // memory byte stream
        pos = 0;
        for(w=0;w<GetMemoryWordCount(m);w++) {
            GetScreenBuffer(m,w,tmp);
            for(i=0;i<GetMemoryWordSize(m);i++) {
                mem_bytes[pos] = tmp[i];
                pos++;
            }
        }
        // save
        pos = 0;
        while(true) {
            bytes_to_write = 16;
            if(bytes_to_write>(mem_bytes_count-pos)) bytes_to_write = mem_bytes_count-pos;
            if(bytes_to_write<1) break;
            // address record
            if(off != ((pos+GetMemoryOffset(m)) >> 16)) {
                off = (pos+GetMemoryOffset(m)) >> 16;
                line[0] = 2;
                line[1] = 0;
                line[2] = 0;
                line[3] = 4;
                line[4] = (off>>8) & 0xFF;
                line[5] = off & 0xFF;
                crc = 0;
                for(i=0;i<6;i++) crc += line[i];
                crc = (~crc) + 1;
                line[6] = crc;
                out.Write(":", 1);
                for(i=0;i<7;i++) {
                    ToStr(line[i],b);
                    out.Write(b, 2);
                }
                out.Write("\r\n", 2);
            }
            // data record
            line[0] = bytes_to_write;
            line[1] = ((pos+GetMemoryOffset(m)) >> 8) & 0xFF;
            line[2] = (pos+GetMemoryOffset(m))& 0xFF;
            line[3] = 0x00;
            for(i=0;i<bytes_to_write;i++) line[4+i]=mem_bytes[pos+i];
            crc = 0;
            for(i=0;i<bytes_to_write+4;i++) crc += line[i];
            crc = (~crc) + 1;
            line[bytes_to_write+4] = crc;
            pos += bytes_to_write;
            out.Write(":", 1);
            for(i=0;i<bytes_to_write+5;i++) {
                ToStr(line[i],b);
                out.Write(b, 2);
            }
            out.Write("\r\n", 2);
        }
        delete mem_bytes;
    }
    // file end
    out.Write(":00000001FF\r\n", 13);
    delete tmp;
    AddLog(STR_SAVE_OK + filename);
}

void Device::OpenAsBinary(wxString filename)
{
    int m,w;
    unsigned char* tmp;
    wxFFileInputStream in(filename);
    if(!(in.IsOk())) {
        wxMessageBox(_T(STR_OPEN_ERR_MESSAGE), _T(STR_OPEN_ERR_CAPTION), wxOK | wxICON_ERROR);
        return;
    }
    tmp = new unsigned char[max_bytes_per_word];
    for(m=0;m<GetMemoriesCount();m++) {
        for(w=0;w<GetMemoryWordCount(m);w++) {
            in.Read(tmp, GetMemoryWordSize(m));
            if(in.LastRead() != GetMemoryWordSize(m)) {
                wxMessageBox(_T(STR_OPEN_ERR_MESSAGE), _T(STR_OPEN_ERR_CAPTION), wxOK | wxICON_ERROR);
                delete tmp;
                return;
            }
            SetScreenBuffer(m,w,tmp);
        }
    }
    delete tmp;
    AddLog(STR_OPEN_OK + filename);
}

void Device::OpenAsHex(wxString filename)
{
    wxTextFile tf(filename);
    tf.Open();
    if(!(tf.IsOpened())) {
        wxMessageBox(_T(STR_OPEN_ERR_MESSAGE), _T(STR_OPEN_ERR_CAPTION), wxOK | wxICON_ERROR);
        return;
    }
    wxString str;
    int i,a,m,w,b;
    int a_min, a_max;
    unsigned int off;
    unsigned char* buf;
    unsigned char* tmp;
    off = 0;
    buf = new unsigned char[300];
    tmp = new unsigned char[max_bytes_per_word];
    for( str = tf.GetFirstLine(); !tf.Eof(); str = tf.GetNextLine()) {
        if(str.Len()<11) continue;
        if(str[0]!=':') continue;
        for(i=0;i<(((int)str.Len())-1)/2;i++) {
            buf[i]=(ToHex(str[1+i*2]) << 4) | ToHex(str[1+i*2+1]);
        }
        if(buf[3]==0) { // data
            for(i=0;i<buf[0];i++) {
                a = (int)off + (((int)(buf[1]))<<8) + ((int)buf[2]) + i;
                // memory
                for(m=0;m<GetMemoriesCount();m++) {
                    a_min = GetMemoryOffset(m);
                    a_max = GetMemoryOffset(m)+GetMemoryWordCount(m)*GetMemoryWordSize(m)-1;
                    if((a>=a_min) && (a<=a_max)) break;
                }
                if(m<GetMemoriesCount()) {
                    w = (a-GetMemoryOffset(m)) / GetMemoryWordSize(m);
                    b = (a-GetMemoryOffset(m)) % GetMemoryWordSize(m);
                    GetScreenBuffer(m,w,tmp);
                    tmp[b] = buf[4+i];
                    SetScreenBuffer(m,w,tmp);
                }
            }
        }
        else if(buf[3]==1) { // end
            break;
        }
        else if((buf[3]==2) && (buf[0]==2)) { // offset
            off = (buf[4]<<12) | (buf[5]<<4);
        }
        else if((buf[3]==4) && (buf[0]==2)) { // offset
            off = (buf[4]<<24) | (buf[5]<<16);
        }
    }
    delete buf;
    delete tmp;
    AddLog(STR_OPEN_OK + filename);
}

bool Device::OpenProg()
{
    prog->QOpenProg(prog->preferred_prog_sn);
    int status = ReadStatus("");
    if(status == OPEN_OK) return true;
    else return false;
}

bool Device::CloseProg()
{
    prog->QCloseProg();
    int status = ReadStatus("");
    if(status == CLOSE_OK) return true;
    else return false;
}

int Device::ReadStatus(wxString message)
{
    int status = prog->AGetBlocking();
    wxString codestring = prog->ReturnCodeString(status);
    if(message.IsEmpty()) {
        AddLog(codestring);
    }
    else {
        AddLog(message+": "+codestring);
    }
    return status;
}

