/////////////////////////////////////////////////////////////////////////////
// Name:        mframe.h
// Purpose:     main programmer frame
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

#ifndef _MFRAME_H
#define _MFRAME_H

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/fileconf.h>
#include <wx/mdi.h>
#include <wx/filedlg.h>

// user header files
#include "common.h"
#include "string_const.h"
#include "programmer.h"
#include "device.h"
#include "device_24LC02.h"
#include "device_93LC46B.h"
#include "device_M25P80.h"
#include "device_DS18B20.h"

// ----------------------------------------------------------------------------
// Dynamic Arrays
// ----------------------------------------------------------------------------
#include <wx/dynarray.h>

WX_DECLARE_OBJARRAY(Device, ArrayOfDevices);

// ----------------------------------------------------------------------------
// class model
// ----------------------------------------------------------------------------

// Define a new frame type: this is going to be main frame
class MainFrame : public wxMDIParentFrame
{
public:
    // ctor(s)
    MainFrame(wxFileConfig *cfg, Programmer *prg, const wxString& title, const wxPoint& pos, const wxSize& size, long style);
    // event handlers (these functions should _not_ be virtual)
    void OnCloseQuery(wxCloseEvent& event);
    void OnQuit(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);
    void OnDeviceSelect(wxCommandEvent& evt);
    void OnShowAllClick(wxCommandEvent& evt);
    void OnHideAllClick(wxCommandEvent& evt);
    void OnActionWindowShowClick(wxCommandEvent& evt);
    void OnBitAliasWindowShowClick(wxCommandEvent& evt);
    void OnDeviceHelpWindowShowClick(wxCommandEvent& evt);
    void OnLogWindowShowClick(wxCommandEvent& evt);
    void OnSettingsWindowShowClick(wxCommandEvent& evt);
    void OnMemoryWindowShowClick(wxCommandEvent& evt);
    void OnActionClick(wxCommandEvent& evt);
    void OnBitAliasWindowValueChanged(wxPropertyGridEvent& evt);
    void OnNewFileClick(wxCommandEvent& evt);
    void OnOpenFileClick(wxCommandEvent& evt);
    void OnSaveFileAsClick(wxCommandEvent& evt);
    void OnLogClearClick(wxCommandEvent& evt);
    void OnLogSaveClick(wxCommandEvent& evt);
    void OnRefreshProgListClick(wxCommandEvent& evt);
    void OnProgrammerClick(wxCommandEvent& evt);
    //
private:
    // variables
    wxFileConfig *config;
    Programmer *prog;
    ArrayOfDevices devices;
    wxMenu *menuDevice;
    wxMenu *menuAction;
    wxMenu *menuWindow;
    wxMenu *menuProg;
    Device *current_device;
    wxArrayInt prog_list;
    // functions
    void SelectDevice(int device_index);
    void DeselectDevice();
    void RefreshProgList();
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif /* _MFRAME_H*/
