/////////////////////////////////////////////////////////////////////////////
// Name:        programmer.h
// Purpose:     programmer interface
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

#ifndef _PROGRAMMER_H
#define _PROGRAMMER_H

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wx/dynlib.h"

// user header files
#include "string_const.h"

// ----------------------------------------------------------------------------
// programmer constants
// ----------------------------------------------------------------------------

/* QSetPins constants */
#define PINS_HIZ    0x01
#define PINS_LO     0x02
#define PINS_HI     0x03
#define PINS_D_BIT  0x00
#define PINS_C_BIT  0x02
#define PINS_I_BIT  0x04
#define PINS_L_BIT  0x06
#define PINS_P_BIT  0x08
#define PINS_R_BIT  0x0A
#define PINS_S_BIT  0x0C
#define PINS_T_BIT  0x0E

/* QShift... constants */
#define SHIFT_OUTIN_PIND    0x00
#define SHIFT_OUTIN_PINI    0x02
#define SHIFT_OUTIN_PINL    0x03
#define SHIFT_OUTIN_PINP    0x04
#define SHIFT_OUTIN_PINR    0x05
#define SHIFT_OUTIN_PINS    0x06
#define SHIFT_OUTIN_PINT    0x07
#define SHIFT_MODE0         0x00
#define SHIFT_MODE1         0x01
#define SHIFT_MODE2         0x02
#define SHIFT_MODE3         0x03

/* QSetShiftSpeed constants */
#define SHIFT_CLK_15000kHz  1
#define SHIFT_CLK_10000kHz  2
#define SHIFT_CLK_7500kHz   3
#define SHIFT_CLK_6000kHz   4
#define SHIFT_CLK_5000kHz   5
#define SHIFT_CLK_3750kHz   6
#define SHIFT_CLK_3330kHz   7
#define SHIFT_CLK_3000kHz   8
#define SHIFT_CLK_2500kHz   9
#define SHIFT_CLK_2000kHz   10
#define SHIFT_CLK_1500kHz   11
#define SHIFT_CLK_1000kHz   12
#define SHIFT_CLK_750kHz    13
#define SHIFT_CLK_600kHz    14
#define SHIFT_CLK_500kHz    15
#define SHIFT_CLK_400kHz    16
#define SHIFT_CLK_375kHz    17
#define SHIFT_CLK_333kHz    18
#define SHIFT_CLK_300kHz    19
#define SHIFT_CLK_250kHz    20
#define SHIFT_CLK_200kHz    21
#define SHIFT_CLK_150kHz    22
#define SHIFT_CLK_120kHz    23
#define SHIFT_CLK_100kHz    24
#define SHIFT_CLK_75kHz     25
#define SHIFT_CLK_60kHz     26
#define SHIFT_CLK_50kHz     27
#define SHIFT_CLK_40kHz     28
#define SHIFT_CLK_37kHz     29
#define SHIFT_CLK_33kHz     30
#define SHIFT_CLK_30kHz     31

/* QSetPullUpDowns constants */
#define PULLDOWN    0x01
#define PULLUP      0x02
#define D_PULL      0x00
#define C_PULL      0x02
#define I_PULL      0x04
#define L_PULL      0x06
#define S_PULL      0x08
#define T_PULL      0x0A
#define P_PULL      0x0C
#define R_PULL      0x0E

/* QI2CSetSpeed constants */
#define I2C_CLK_100kHz  0x00
#define I2C_CLK_400kHz  0x01
#define I2C_CLK_1MHz    0x02

/* QSetActiveLED constants */
#define LED_ACT_OFF             0x00
#define LED_ACT_Y               0x01
#define LED_ACT_R               0x02
#define LED_ACT_Y_BLINK         0x03
#define LED_ACT_R_BLINK         0x04
#define LED_ACT_Y_FAST_BLINK    0x05
#define LED_ACT_R_FAST_BLINK    0x06
#define LED_ACT_YR_FAST_BLINK   0x07

/* return codes */
#define OPEN_OK                 0x10000
#define OPEN_NOTFOUND           0x10001
#define OPEN_CANNOTOPEN         0x10002
#define OPEN_ALREADYOPEN        0x10003
#define OPEN_BADDRIVERVERSION   0x10004
#define CLOSE_OK                0x20000
#define CLOSE_CANNOTCLOSE       0x20001
#define POWERON_OK              0x30000
#define POWERON_OCURR           0x30001
#define POWERON_WRONG_LEVEL     0x30002
#define GETPINS_CODE            0x40000 /* ORed with GETPINS_PINx */
#define GETPINS_PIND            0x01
#define GETPINS_PINC            0x02
#define GETPINS_PINI            0x04
#define GETPINS_PINL            0x08
#define GETPINS_PINP            0x10
#define GETPINS_PINR            0x20
#define GETPINS_PINS            0x40
#define GETPINS_PINT            0x80
#define OK                      0x50000
#define NOT_OPENED              0x50001
#define WRONG_INPUT             0x50002
#define SHIFT_BYTE_OUTIN_CODE   0x60000
#define SUPPLY_VOLTAGE_CODE     0x70000
#define VPP_OK                  0x80000
#define VPP_OCURR               0x80001
#define VPP_WRONG_LEVEL         0x80002
#define GO_BUTTON_NOT_PRESSED   0x90000
#define GO_BUTTON_PRESSED       0x90001
#define SHIFT_BITS_OUTIN_CODE   0xA0000
#define _1WIRE                  0xB0000
#define _1WIRE_PRESENT          0xB0100
#define _1WIRE_NOT_PRESENT      0xB0200
#define I2C_CODE                0xC0000
#define I2C_ACK                 0xC0100
#define I2C_NACK                0xC0200
#define FATAL_OVERCURRENTVDD    0x01
#define FATAL_OVERCURRENTVPP    0x02
#define FATAL_OVERVOLTAGEVDD    0x04
#define FATAL_OTHER             0x08

/* DLL loading */
#define FORTE_DLL_LOAD_OK               0
#define FORTE_DLL_LOAD_FAIL_NAME        1
#define FORTE_DLL_LOAD_FAIL_FUNCTION    2

/* DLL exported functions */

typedef void (CALLBACK* LPFNDLLAClearFatalError)(void);
typedef int  (CALLBACK* LPFNDLLAGet)(int *answer);
typedef int  (CALLBACK* LPFNDLLAGetBlock)(int *buf, int count, int *count_returned);
typedef int  (CALLBACK* LPFNDLLAGetBlocking)(void);
typedef void (CALLBACK* LPFNDLLAGetProgList)(int *sn_list, int count, int *count_returned);
typedef int  (CALLBACK* LPFNDLLAGetStatus)(int *NumberOfAnswers);
typedef void (CALLBACK* LPFNDLLQ1WireInit)(void);
typedef void (CALLBACK* LPFNDLLQ1WireReadByte)(void);
typedef void (CALLBACK* LPFNDLLQ1WireWriteByte)(int data, int strong_pullup_time_us);
typedef void (CALLBACK* LPFNDLLQCheckGoButton)(void);
typedef void (CALLBACK* LPFNDLLQCheckSupplyVoltage)(void);
typedef void (CALLBACK* LPFNDLLQCloseProg)(void);
typedef void (CALLBACK* LPFNDLLQDelay)(int delayus);
typedef void (CALLBACK* LPFNDLLQDelay_ns)(int delayns);
typedef void (CALLBACK* LPFNDLLQGetPins)(void);
typedef void (CALLBACK* LPFNDLLQI2CReadByte)(int ACK);
typedef void (CALLBACK* LPFNDLLQI2CSetSpeed)(int speed);
typedef void (CALLBACK* LPFNDLLQI2CStart)(int UseIntPullUps);
typedef void (CALLBACK* LPFNDLLQI2CStop)(void);
typedef void (CALLBACK* LPFNDLLQI2CWriteByte)(int databyte);
typedef void (CALLBACK* LPFNDLLQOpenProg)(int sn);
typedef void (CALLBACK* LPFNDLLQPoweroffVdd)(void);
typedef void (CALLBACK* LPFNDLLQPoweroffVpp)(void);
typedef void (CALLBACK* LPFNDLLQPoweronVdd)(int delayus, int Voltage_mV);
typedef void (CALLBACK* LPFNDLLQPoweronVpp)(int Voltage_mV);
typedef void (CALLBACK* LPFNDLLQSetActiveLED)(int led);
typedef void (CALLBACK* LPFNDLLQSetGPIOAnswer)(int answer);
typedef void (CALLBACK* LPFNDLLQSetPins)(int pins);
typedef void (CALLBACK* LPFNDLLQSetPullUpDowns)(int pullupdowns);
typedef void (CALLBACK* LPFNDLLQSetShiftSpeed)(int speed);
typedef void (CALLBACK* LPFNDLLQShiftBits)(int data, int mode, int bits_count);
typedef void (CALLBACK* LPFNDLLQShiftBits_OutIn)(int data, int mode, int InputPin, int bits_count);
typedef void (CALLBACK* LPFNDLLQShiftByte)(int databyte, int mode);
typedef void (CALLBACK* LPFNDLLQShiftByte_OutIn)(int databyte, int mode, int InputPin);
typedef void (CALLBACK* LPFNDLLQShiftBytes)(int *buf, int mode, int count);
typedef void (CALLBACK* LPFNDLLQShiftBytes_OutIn)(int *buf, int mode, int InputPin, int count);
typedef void (CALLBACK* LPFNDLLQShiftBytes_In)(int mode, int InputPin, int count);

// ----------------------------------------------------------------------------
// class model
// ----------------------------------------------------------------------------

class Programmer
{
public:
    // Constructor and destructor
    Programmer();
    // functions
    wxString ReturnCodeString(int status);
    int LoadDll();
    //
    LPFNDLLAClearFatalError     AClearFatalError;
    LPFNDLLAGet                 AGet;
    LPFNDLLAGetBlock            AGetBlock;
    LPFNDLLAGetBlocking         AGetBlocking;
    LPFNDLLAGetProgList         AGetProgList;
    LPFNDLLAGetStatus           AGetStatus;
    LPFNDLLQ1WireInit           Q1WireInit;
    LPFNDLLQ1WireReadByte       Q1WireReadByte;
    LPFNDLLQ1WireWriteByte      Q1WireWriteByte;
    LPFNDLLQCheckGoButton       QCheckGoButton;
    LPFNDLLQCheckSupplyVoltage  QCheckSupplyVoltage;
    LPFNDLLQCloseProg           QCloseProg;
    LPFNDLLQDelay               QDelay;
    LPFNDLLQDelay_ns            QDelay_ns;
    LPFNDLLQGetPins             QGetPins;
    LPFNDLLQI2CReadByte         QI2CReadByte;
    LPFNDLLQI2CSetSpeed         QI2CSetSpeed;
    LPFNDLLQI2CStart            QI2CStart;
    LPFNDLLQI2CStop             QI2CStop;
    LPFNDLLQI2CWriteByte        QI2CWriteByte;
    LPFNDLLQOpenProg            QOpenProg;
    LPFNDLLQPoweroffVdd         QPoweroffVdd;
    LPFNDLLQPoweroffVpp         QPoweroffVpp;
    LPFNDLLQPoweronVdd          QPoweronVdd;
    LPFNDLLQPoweronVpp          QPoweronVpp;
    LPFNDLLQSetActiveLED        QSetActiveLED;
    LPFNDLLQSetGPIOAnswer       QSetGPIOAnswer;
    LPFNDLLQSetPins             QSetPins;
    LPFNDLLQSetPullUpDowns      QSetPullUpDowns;
    LPFNDLLQSetShiftSpeed       QSetShiftSpeed;
    LPFNDLLQShiftBits           QShiftBits;
    LPFNDLLQShiftBits_OutIn     QShiftBits_OutIn;
    LPFNDLLQShiftByte           QShiftByte;
    LPFNDLLQShiftByte_OutIn     QShiftByte_OutIn;
    LPFNDLLQShiftBytes          QShiftBytes;
    LPFNDLLQShiftBytes_OutIn    QShiftBytes_OutIn;
    LPFNDLLQShiftBytes_In       QShiftBytes_In;
    //variables 
    int preferred_prog_sn;
    //
private: 
    // variables
    wxDynamicLibrary fortedll;
    static int KEYS[];
    static char* CODES[];
    // functions
    //
};

#endif /* _PROGRAMMER_H*/
