#
# ASIX FORTE 1-wire thermometer DS18B20 example
# distributed as-is, without any warranties
# http://www.asix.net/
#
# version 1.0, Dec 2016
#
# Programmer connection table (parasite-powered mode): 
# DS18B20 <->  FORTE
# GND     <->  GND
# DQ      <->  P
# VDD     <->  GND
#
# Programmer connection table (external supply mode): 
# DS18B20 <->  FORTE
# GND     <->  GND
# DQ      <->  P
# VDD     <->  VDD
#

from forte import *

def ReadStatus(message = ""):
    status = AGetBlocking()
    if message != "":
        print(message + ": " + RETURNCODE[status])
    else:
        print(RETURNCODE[status])

def main():
    LoadForteDll()
    QOpenProg()
    ReadStatus()
    
    # set supply voltage to 3.3V
    QPoweronVdd(20000, 3300)
    ReadStatus("PoweronVdd")

    # enable 2k4 pull-up
    QSetPullUpDowns((PULLUP << P_PULL))
    ReadStatus("SetPullUpDowns")

    # switch off the answer of the output GPIO functions and delay
    QSetGPIOAnswer(False)
    ReadStatus("SetGPIOAnswer")
    
    # measure temperature
    Q1WireInit()
    # skip ROM command
    Q1WireWriteByte(0xCC, 0)
    # convert T command, with strong pull-up for 750ms, which is the conversion time
    Q1WireWriteByte(0x44, 750000)
    
    # read the measured value
    Q1WireInit()
    # skip ROM command
    Q1WireWriteByte(0xCC, 0)
    # read scratchpad, read only the two first bytes, which is the temperature
    Q1WireWriteByte(0xBE, 0)
    Q1WireReadByte()
    Q1WireReadByte()

    # get results of the two inits
    status1 = AGetBlocking()
    status2 = AGetBlocking()
    # get the measured data
    data_lo = AGetBlocking() & 0xFF
    data_hi = AGetBlocking() & 0xFF
    
    if (status1 == ONE_WIRE_PRESENT) and (status2 == ONE_WIRE_PRESENT):
        data = (data_hi << 8) | data_lo
        # convert to 16-bit signed
        if data >= 0x8000:
            data = data - 0x10000
        temperature = 0.0625*data
        print("Temperature: %.1f degrees C"%(temperature))
    else:
        print("1Wire: ERROR, device not present.")
    
    # enable the GPIO answer
    QSetGPIOAnswer(True)
    ReadStatus("SetGPIOAnswer")

    QCloseProg()
    ReadStatus()


if __name__ == "__main__":
    main()
