#
# ASIX FORTE serial FLASH M25P80 example
# distributed as-is, without any warranties
# http://www.asix.net/
#
# version 1.0, Dec 2016
#
# Programmer connection table: 
# M25P80  <->  FORTE
# S#      <->  P
# VCC     <->  VDD
# VSS     <->  GND
# DQ0     <->  D
# C       <->  C
# DQ1     <->  I
# W#      <->  VDD
# HOLD#   <->  VDD
#

from forte import *

def ReadStatus(message = ""):
    status = AGetBlocking()
    if message != "":
        print(message + ": " + RETURNCODE[status])
    else:
        print(RETURNCODE[status])
        
def PrintHex(data):
    for j in range(len(data)/16):
        s = ""
        for i in range(16):
            s += "%02X "%(data[j*16+i])
        print(s)
    j = len(data)/16
    s = ""
    for i in range(len(data)%16):
        s += "%02X "%(data[j*16+i])
    if s != "":
        print(s)

def LsbToMsb(byte):
    x = 0
    for i in range(8):
        x = x | (((byte >> i) & 1) << (7-i))
    return x;

def main():
    LoadForteDll()
    QOpenProg()
    ReadStatus()
    
    # initiate
    QSetGPIOAnswer(False)
    ReadStatus("SetGPIOAnswer")
    QSetActiveLED(LED_ACT_Y_FAST_BLINK)
    ReadStatus("SetActiveLED")
    QPoweronVdd(20000, 3300) # supply voltage 3.3V
    ReadStatus("PoweronVdd")
    QSetShiftSpeed(SHIFT_CLK_10000kHz)
    QSetPins((PINS_HI << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)) # S#=1, C=0, DQ0=0
    QDelay(100)
    QSetPins((PINS_LO << PINS_P_BIT)) # S#=0
    QDelay(1)
    
    # command code and address
    QShiftByte(0xC0, SHIFT_MODE0)
    QShiftByte(0x00, SHIFT_MODE0)
    QShiftByte(0x00, SHIFT_MODE0)
    QShiftByte(0x00, SHIFT_MODE0)
    # read memory
    mem_bytes_count = 1024*1024 # total read bytes
    block_in_size = 512 # bytes in one QShiftBytes_In()
    buffer_size = 16384 # bytes in one AGetBlock()
    print("Reading %d bytes ..."%mem_bytes_count)
    data = []
    data_count = 0
    while data_count < mem_bytes_count:
        buffer_count = 0
        while (buffer_count < buffer_size) and (data_count+buffer_count < mem_bytes_count):
            n = mem_bytes_count - data_count - buffer_count
            if n > block_in_size:
                n = block_in_size
            QShiftBytes_In(SHIFT_MODE0, SHIFT_OUTIN_PINI, n)
            buffer_count += n
        while True:
            status, received_bytes = AGetStatus()
            if not status:
                break
            if received_bytes >= buffer_count:
                break
        if not status:
            break
        data += AGetBlock(buffer_count)
        data_count += buffer_count
    #
    QSetPins((PINS_HI << PINS_P_BIT)) # S#=1
    QSetActiveLED(LED_ACT_OFF)
    ReadStatus("SetActiveLED")

    # take 8-bit data and print
    if status:
        for i in range(len(data)):
            data[i] = LsbToMsb(data[i] & 0xFF)
        print("First 128 data bytes:")
        PrintHex(data[:128])
        print("Last 128 data bytes:")
        PrintHex(data[-128:])
    else:
        print("Data read failed")

    QSetGPIOAnswer(True)
    ReadStatus("SetGPIOAnswer")
    QCloseProg()
    ReadStatus()


if __name__ == "__main__":
    main()
